/*******************************************************************************
***  This file is part of Qt Mocks.                                          ***
***                                                                          ***
***  Copyright (C) 2018, 2019, 2020, 2021                                    ***
***  John Schneiderman <Licencing _AT_ Schneiderman _DOT_ me>                ***
***                                                                          ***
***  This program is free software: you can redistribute it and/or modify it ***
***  under the terms of the GNU Lesser General Public License as published   ***
***  by the Free Software Foundation, either version 3 of the License, or    ***
***  (at your option) any later version.                                     ***
***                                                                          ***
***  This program is distributed in the hope that it will be useful, but     ***
***  WITHOUT ANY WARRANTY; without even the implied warranty of              ***
***  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    ***
***  See the GNU Lesser General Public License for more details.             ***
***                                                                          ***
***  You should have received a copy of the GNU Lesser General Public License***
***  along with this program. If not, see <http://www.gnu.org/licenses/>.    ***
*******************************************************************************/
#include "Matchers.h"
using std::boolalpha;
using std::ostringstream;
using std::string;
using qt_mocks::common_matchers::BooleanMatcher;
using qt_mocks::common_matchers::EqualMatcher;
using qt_mocks::common_matchers::ExpectedValue;
using qt_mocks::common_matchers::Matcher;

#include <algorithm>
using std::min;
#include <cmath>
using std::abs;


bool EqualMatcher<float>::compare(const float& actual) const
{
	Q_ASSERT(this->expected_ != nullptr);
	return qFuzzyCompare(actual, this->expected_->value());
}

string EqualMatcher<float>::description(const float& actual) const
{
	Q_ASSERT(this->expected_ != nullptr);
	ostringstream oss;
	oss << "The single precision floating point actual value, " << actual
			<< ", should be roughly equal to the expected value, "
			<< this->expected_->value() << '.';
	return oss.str();
}

bool EqualMatcher<double>::compare(const double& actual) const
{
	Q_ASSERT(this->expected_ != nullptr);
	return qFuzzyCompare(actual, this->expected_->value());
}

string EqualMatcher<double>::description(const double& actual) const
{
	Q_ASSERT(this->expected_ != nullptr);
	ostringstream oss;
	oss << "The double precision floating point actual value, " << actual
			<< ", should be roughly equal to the expected value, "
			<< this->expected_->value() << '.';
	return oss.str();
}

bool EqualMatcher<long double>::compare(const long double& actual) const
{
	Q_ASSERT(this->expected_ != nullptr);
	return abs(this->expected_->value() - actual) * 10000000000000000000.0L
			<= min(abs(this->expected_->value()), abs(actual));
}

string EqualMatcher<long double>::description(const long double& actual) const
{
	Q_ASSERT(this->expected_ != nullptr);
	ostringstream oss;
	oss << "The quadruple precision floating point actual value, " << actual
			<< ", should be roughly equal to the expected value, "
			<< this->expected_->value() << '.';
	return oss.str();
}

BooleanMatcher::BooleanMatcher(const bool expected) :
	Matcher<bool>{ExpectedValue<bool>{expected}}
{}

bool BooleanMatcher::compare(const bool& actual) const
{
	return actual == this->expected_->value();
}

string BooleanMatcher::description(const bool& actual) const
{
	ostringstream oss;
	oss << boolalpha << "The actual Boolean value '" << actual << "' should be '"
		<< this->expected_->value() << "'.";
	return oss.str();
}

BooleanMatcher qt_mocks::isFalse()
{
	return BooleanMatcher{false};
}

BooleanMatcher qt_mocks::isTrue()
{
	return BooleanMatcher{true};
}
