/*******************************************************************************
***  This file is part of Qt Mocks.                                          ***
***                                                                          ***
***  Copyright (C) 2018, 2019, 2020, 2021                                    ***
***  John Schneiderman <Licencing _AT_ Schneiderman _DOT_ me>                ***
***                                                                          ***
***  This program is free software: you can redistribute it and/or modify it ***
***  under the terms of the GNU Lesser General Public License as published   ***
***  by the Free Software Foundation, either version 3 of the License, or    ***
***  (at your option) any later version.                                     ***
***                                                                          ***
***  This program is distributed in the hope that it will be useful, but     ***
***  WITHOUT ANY WARRANTY; without even the implied warranty of              ***
***  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    ***
***  See the GNU Lesser General Public License for more details.             ***
***                                                                          ***
***  You should have received a copy of the GNU Lesser General Public License***
***  along with this program. If not, see <http://www.gnu.org/licenses/>.    ***
*******************************************************************************/
#include <QList>
using ::QList;
#include <QMap>
using ::QMap;
#include <QTest>
// using QTEST_MAIN

#include <ostream>
using std::ostream;
#include <map>
using std::map;
#include <set>
using std::set;
#include <string>
using std::string;
#include <vector>
using std::vector;

#include <qt-mocks/Matchers.h>
using qt_mocks::common_matchers::BooleanMatcher;
using qt_mocks::allOf;
using qt_mocks::anyOf;
using qt_mocks::between;
using qt_mocks::betweenBeginInclusive;
using qt_mocks::betweenEndInclusive;
using qt_mocks::betweenInclusive;
using qt_mocks::equals;
using qt_mocks::greaterThan;
using qt_mocks::greaterThanEqual;
using qt_mocks::isEmpty;
using qt_mocks::isTrue;
using qt_mocks::isFalse;
using qt_mocks::lessThan;
using qt_mocks::lessThanEqual;
using qt_mocks::memberFunction;
using qt_mocks::memberVariable;
using qt_mocks::not_;
using qt_mocks::notEquals;


namespace qt_mocks
{
namespace tests
{

struct Test
{
	qint32 memberAccessor() const
	{
		return 15;
	}
	bool operator==(const Test& other) const
	{
		return this->member_ == other.member_;
	}

	qint32 member_;
};
struct TestConstructor
{
	friend ostream& operator<<(ostream& o, const TestConstructor& tc);

	explicit TestConstructor(const qint32 value) :
		member_{value}
	{}
	qint32 memberAccessor() const
	{
		return this->member_;
	}
	bool operator==(const TestConstructor& other) const
	{
		return this->member_ == other.member_;
	}

	qint32 member_;
};
ostream& operator<<(ostream& o, const TestConstructor& tc)
{
	return o << '(' << tc.member_ << ')';
}

struct MatchersUT : QObject
{
	Q_OBJECT

private slots:
	//{ Relational Tests

	void expectFalse_booleanFalseValue_ShouldBeTrue()
	{
		QVERIFY_FALSE(false);
	}
	void expectTrue_booleanTrueValue_ShouldBeTrue()
	{
		QVERIFY_TRUE(true);
	}
	void expectNot_notTrue_ShouldBeFalse()
	{
		QVERIFY_THAT(false, not_(BooleanMatcher{true}));
		QVERIFY_THAT(false, not_(isTrue()));
	}
	void expectNot_notFalse_ShouldBeTrue()
	{
		QVERIFY_THAT(true, not_(BooleanMatcher{false}));
		QVERIFY_THAT(true, not_(isFalse()));
	}
	void expectEqual_twoCmpTwo_ShouldBeTrue()
	{
		QVERIFY_THAT(2, equals(2));
	}
	void expectEqual_twoCmpTwo_ShouldBeCalledOnlyOnce()
	{
		qint32 val{0};
		const auto adder{
			[&val] ()
			{
				val += 2;
				return val;
			}

		};
		QVERIFY_THAT(adder(), equals(2));
	}
	void expectEqual_twoPointFourFiveCmpTwoPointFourFive_ShouldBeTrue()
	{
		QVERIFY_THAT(2.45f, equals(2.45f));
		QVERIFY_THAT(2.45, equals(2.45));
		QVERIFY_THAT(2.45L, equals(2.45L));
	}
	void expectEqual_twoCmpFour_ShouldBeFalse()
	{
		QVERIFY_THAT(2, not_(equals(4)));
	}
	void expectNotEqual_twoCmpFour_ShouldBeFalse()
	{
		QVERIFY_THAT(2, notEquals(4));
		QVERIFY_THAT(2, not_(notEquals(2)));
	}
	void expectLessThan_twoCmpFour_ShouldBeTrue()
	{
		QVERIFY_THAT(2, lessThan(4));
	}
	void expectGreaterThan_twoCmpNegOne_ShouldBeTrue()
	{
		QVERIFY_THAT(2, greaterThan(-1));
	}
	void expectLessThanEqual_twoCmpFour_ShouldBeTrue()
	{
		QVERIFY_THAT(2, lessThanEqual(4));
	}
	void expectLessThanEqual_twoCmpTwo_ShouldBeTrue()
	{
		QVERIFY_THAT(2, lessThanEqual(2));
	}
	void expectGreaterThanEqual_twoCmpNegOne_ShouldBeTrue()
	{
		QVERIFY_THAT(2, greaterThanEqual(-1));
	}
	void expectGreaterThanEqual_twoCmpTwo_ShouldBeTrue()
	{
		QVERIFY_THAT(2, greaterThanEqual(2));
	}
	void expectBetween_oneCmpThree_ShouldBeTrue()
	{
		QVERIFY_THAT(2, between(1, 3));
		QVERIFY_THAT(2, not_(between(1, 2)));
		QVERIFY_THAT(2, not_(between(2, 3)));
	}
	void expectBetweenInclusive_twoCmpFour__ShouldBeTrue()
	{
		QVERIFY_THAT(2, betweenInclusive(1, 2));
		QVERIFY_THAT(2, betweenInclusive(2, 2));
		QVERIFY_THAT(2, betweenInclusive(2, 3));
		QVERIFY_THAT(2, betweenInclusive(1, 3));
		QVERIFY_THAT(2, not_(between(3, 4)));
	}
	void expectBetweenInclusiveBegining_twoCmpThree_ShouldBeTrue()
	{
		QVERIFY_THAT(2, betweenBeginInclusive(1, 3));
		QVERIFY_THAT(2, not_(betweenBeginInclusive(2, 2)));
		QVERIFY_THAT(2, betweenBeginInclusive(2, 3));
	}
	void expectBetweenInclusiveEnding_twoCmpThree_ShouldBeTrue()
	{
		QVERIFY_THAT(2, betweenEndInclusive(1, 2));
		QVERIFY_THAT(2, not_(betweenEndInclusive(2, 2)));
		QVERIFY_THAT(2, betweenEndInclusive(1, 3));
	}

	//}

	void expectNoThrow_doesNotThrow_ShouldPass()
	{
		const auto noThrow{
			[] ()
			{
				return 0;
			}
		};
		QVERIFY_NO_THROW(noThrow());
	}

#	ifdef _WIN32
#		pragma warning(push)
#		pragma warning (disable : 4702 )
#		// Justification: Still need to implement for testing.
#	endif
	void expectNoThrow_doesThrowStdException_ShouldFail()
	{
		const auto doesThrow{
			[] ()
			{
				throw std::logic_error{"Testing throw."};
			}
		};
		QSKIP("This is here to be able to perform manual testing it can be automated.");
		QVERIFY_NO_THROW(doesThrow());
	}
	void expectNoThrow_doesThrowUnknownException_ShouldFail()
	{
		const auto doesThrow{
			[] ()
			{
				throw "Testing throw.";
			}
		};
		QSKIP("This is here to be able to perform manual testing it can be automated.");
		QVERIFY_NO_THROW(doesThrow());
	}

#	ifdef _WIN32
#		pragma warning(pop)
#	endif

	//{ Class Tests

	void noStreamOperator_ShouldUseBytes()
	{
		const Test actual{18};
		const Test expected{18};
		QVERIFY_THAT(actual, equals(expected));
	}
	void memberVariable_ShouldBeValue()
	{
		Test t;
		const qint32 expectedValue{15};
		t.member_ = expectedValue;
		QVERIFY_THAT(t, memberVariable(&Test::member_, equals(expectedValue)));
	}
	void memberAccessor_ShouldBeValue()
	{
		Test t{};
		const qint32 expectedValue{15};
		QVERIFY_THAT(t, memberFunction(&Test::memberAccessor, equals(expectedValue)));
	}
	void memberAccessor_ClassNoDefaultConstructor_ShouldBeValue()
	{
		const qint32 expectedValue{15};
		TestConstructor t{expectedValue};
		QVERIFY_THAT(
			t,
			memberFunction(
				&TestConstructor::memberAccessor,
				equals(expectedValue)
			)
		);
		const vector<qint32> v{1, 10};
		const size_t expectedSize{2};
		QVERIFY_THAT(
			v,
			memberFunction(&vector<qint32>::size, equals(expectedSize))
		);
	}

	//}

	//{ Container Tests

	void equals_OnStdContainer_ShouldBeTrue()
	{
		const set<qint32> expected{1, 2, 3, 4, 5};
		const auto contained{expected};
		QVERIFY_THAT(contained, equals(expected));
	}
	void equals_OnQtContainer_ShouldBeTrue()
	{
		const QList<qint32> expected{1, 2, 3, 4, 5};
		const auto contained{expected};
		QVERIFY_THAT(contained, equals(expected));
	}
	void equals_OnStdAssociativeContainer_ShouldBeTrue()
	{
		const map<string, qint32> expected{{"1", 1}, {"2", 2}, {"3", 3}, {"4", 4}, {"5", 5}};
		const auto contained{expected};
		QVERIFY_THAT(contained, equals(expected));
	}
	void equals_OnQtAssociativeContainer_ShouldBeTrue()
	{
		const QMap<QString, qint32> expected{{"1", 1}, {"2", 2}, {"3", 3}, {"4", 4}, {"5", 5}};
		const auto contained{expected};
		QVERIFY_THAT(contained, equals(expected));
	}
	void isEmpty_NoValues_ShouldBeTrue()
	{
		const vector<qint32> v{};
		QVERIFY_THAT(v, isEmpty<vector<qint32>>());
	}
	void isEmpty_HasValues_ShouldBeFalse()
	{
		const vector<qint32> v{1, 10};
		QVERIFY_THAT(v, not_(isEmpty<vector<qint32>>()));
	}

	//}

	//{ Grouping Tests

	void allOf_OnClass_ShouldBeTrue()
	{
		const qint32 expectedValue{15};
		TestConstructor t{expectedValue};
		QVERIFY_THAT(
			t,
			allOf<TestConstructor>(
				memberVariable(&TestConstructor::member_, equals(expectedValue)),
				memberFunction(
					&TestConstructor::memberAccessor,
					equals(expectedValue)
				),
				equals(t)
			)
		);
	}

	void allOf_OnContainer_ShouldBeTrue()
	{
		const set<qint32> expected{1, 2, 3, 4, 5};
		const auto contained{expected};
		QVERIFY_THAT(
			contained,
			allOf<set<qint32>>(
				memberFunction(&set<qint32>::size, equals(static_cast<size_t>(5))),
				equals(expected)
			)
		);
	}

	void anyOf_ShouldBeTrue()
	{
		QVERIFY_THAT(
			2,
			anyOf<qint32>(
				lessThan(2),
				equals(3),
				greaterThanEqual(2)
			)
		);
	}

	//}
};

}}

QTEST_MAIN(qt_mocks::tests::MatchersUT)
#include "Matchers-unit-tests.moc"
